"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fast_observer_1 = __importDefault(require("fast-observer"));
var isEqual_1 = __importDefault(require("lodash/isEqual"));
var cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
var validation_1 = require("./validation");
var types_1 = require("./types");
/**
 *
 * Form is a singleton which gets passed into an <Input />. It holds realtime
 * data on all errors and values within a collections of inputs.
 */
var Form = /** @class */ (function () {
    function Form(initialValues) {
        if (initialValues === void 0) { initialValues = {}; }
        this.observer = new fast_observer_1.default();
        this.initialValues = {};
        this.inputValidations = {};
        this.values = {};
        this.errors = {};
        this.forceCheck = false;
        this.values = cloneDeep_1.default(initialValues);
        this.initialValues = cloneDeep_1.default(initialValues);
    }
    Form.prototype.inputMode = function (inputName) {
        if (this.forceCheck) {
            return types_1.InputMode.Dirty;
        }
        if (this.initialValues[inputName] === undefined) {
            return types_1.InputMode.Unstable;
        }
        else if (!isEqual_1.default(this.initialValues[inputName], this.values[inputName])) {
            return types_1.InputMode.Dirty;
        }
        return types_1.InputMode.Clean;
    };
    Form.prototype.setInitialValues = function (values) {
        for (var key in values) {
            this.setInitialValue(key, values[key]);
        }
        return this;
    };
    Form.prototype.setValues = function (values) {
        for (var key in values) {
            this.setValue(key, values[key]);
            this.observer.publishFast(key + ".setValue", values[key]);
        }
        return this;
    };
    Form.prototype.forceValidations = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, _i, key;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        this.forceCheck = true;
                        _a = [];
                        for (_b in this.values)
                            _a.push(_b);
                        _i = 0;
                        _c.label = 1;
                    case 1:
                        if (!(_i < _a.length)) return [3 /*break*/, 4];
                        key = _a[_i];
                        return [4 /*yield*/, this.observer.publishFast(key + ".validate")];
                    case 2:
                        _c.sent();
                        _c.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/, this];
                }
            });
        });
    };
    Form.prototype.validate = function (inputName) {
        return __awaiter(this, void 0, void 0, function () {
            var errors;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        errors = [];
                        if (!this.inputValidations[inputName]) return [3 /*break*/, 2];
                        return [4 /*yield*/, validation_1.checkValidations(this.inputValidations[inputName], this.getValue(inputName))];
                    case 1:
                        errors = _a.sent();
                        _a.label = 2;
                    case 2:
                        this.setErrors(inputName, errors);
                        return [2 /*return*/, errors];
                }
            });
        });
    };
    Form.prototype.setValue = function (inputName, value) {
        this.values[inputName] = value;
    };
    Form.prototype.setInitialValue = function (inputName, value) {
        this.initialValues[inputName] = value;
    };
    Form.prototype.setErrors = function (inputName, value) {
        this.errors[inputName] = value;
    };
    Form.prototype.getValue = function (inputName) {
        return cloneDeep_1.default(this.values[inputName]);
    };
    Form.prototype.getValues = function () {
        return cloneDeep_1.default(this.values);
    };
    Form.prototype.getErrors = function (inputName) {
        return cloneDeep_1.default(this.errors[inputName]);
    };
    Form.prototype.setValidations = function (inputName, validations) {
        this.inputValidations[inputName] = validations;
    };
    Form.prototype.hasErrors = function () {
        for (var key in this.errors) {
            var e = this.getErrors(key);
            if (e && e.length) {
                return true;
            }
        }
        return false;
    };
    return Form;
}());
exports.default = Form;
